'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var plugin = 'postcss-property-lookup';
var lookupPattern = /@([a-z-]+)\b/g;
var defaultOptions = {
  logLevel: 'warn'
};

exports['default'] = _postcss2['default'].plugin(plugin, propertyLookup);

function propertyLookup() {
  var _this = this;

  var options = arguments[0] === undefined ? defaultOptions : arguments[0];

  var errorContext = { plugin: plugin };

  var log = ({
    warn: function warn(message, rule, result) {
      rule.warn(result, message);
    },
    error: function error(message, rule) {
      throw rule.error(message, errorContext);
    }
  })[options.logLevel];

  if (!log) {
    throw new Error('Invalid logLevel: ' + options.logLevel);
  }

  return function (root, result) {
    root.walkRules(function (rule) {
      eachDecl(rule, function (decl) {
        if (decl.value.indexOf('@') === -1) {
          return;
        }
        decl.value = decl.value.replace(lookupPattern, resolveLookup.bind(_this, rule));
      });
    });

    function resolveLookup(rule, orig, prop) {
      var resolvedValue = closest(rule, prop);

      if (!resolvedValue) {
        log('Unable to find property ' + orig + ' in ' + rule.selector, rule, result);
      }

      return resolvedValue;
    }

    function closest(_x2, _x3) {
      var _this2 = this;

      var _again = true;

      _function: while (_again) {
        var container = _x2,
            prop = _x3;
        resolvedValue = undefined;
        _again = false;

        if (!container) {
          return '';
        }
        var resolvedValue = undefined;

        eachDecl(container, function (decl) {
          if (decl.prop === prop) {
            resolvedValue = decl.value;
          }
        });

        if (!resolvedValue) {
          _x2 = container.parent;
          _x3 = prop;
          _again = true;
          continue _function;
        }

        if (resolvedValue.indexOf('@') === -1) {
          return resolvedValue;
        }

        return resolvedValue.replace(lookupPattern, resolveLookup.bind(_this2, container));
      }
    }
  };
}

function eachDecl(container, callback) {
  container.each(function (node) {
    if (node.type === 'decl') {
      callback(node);
    }
    // Recurse through child declarations of a media rule
    if (node.type === 'atrule') {
      eachDecl(node, callback);
    }
  });
}
module.exports = exports['default'];